/**
 *	@file color.c Color: manipulao de cores.
 *
 *	@author 
 *			- Maira Noronha
 *			- Thiago Bastos
 *
 *	@date
 *			Criado em:			1 de Dezembro de 2002
 *			ltima Modificao:	22 de Janeiro de 2003
 *
 *	@version 2.0
 */

#include "color.h"

#include <stdio.h>
#include <math.h>


/************************************************************************/
/* Constantes Privadas                                                  */
/************************************************************************/
#define MAX( a, b ) ( ( a > b ) ? a : b )
#define MIN( a, b ) ( ( a < b ) ? a : b )
#define CLAMP( intensity ) MAX( 0.0, MIN( 1.0, intensity ) )


/************************************************************************/
/* Definies das Funes Exportadas                                    */
/************************************************************************/
Color colorCreate3b( unsigned char red, unsigned char green, unsigned char blue )
{
	Color color= { ( red / 255.0 ), ( green / 255.0 ), ( blue / 255.0 ) };

	return color;
}

Color colorAddition( Color c1, Color c2 )
{
	Color color = { CLAMP( c1.red + c2.red ), CLAMP( c1.green + c2.green ), CLAMP( c1.blue + c2.blue ) };

	return color;
}

Color colorContribute( Color c1, Color c2 )
{
	Color color = { c1.red + c2.red, c1.green + c2.green, c1.blue + c2.blue };

	return color;
}

Color colorAdd( Color c1, Color c2 )
{
	Color color = { ( c1.red + c2.red ), ( c1.green + c2.green ), ( c1.blue + c2.blue ) };

	return color;
}

Color colorScale( double s, Color c )
{
	Color color = { CLAMP( s * c.red ), CLAMP( s * c.green ), CLAMP( s * c.blue ) };

	return color;
}

Color colorMultiplication( Color c1, Color c2 )
{
	Color color = { ( c1.red * c2.red ), ( c1.green * c2.green ), ( c1.blue * c2.blue ) };

	return color;
}

Color colorReflection( double s, Color l, Color m )
{
	Color color = { CLAMP( s * ( l.red * m.red ) ), CLAMP( s * ( l.green * m.green ) ), CLAMP( s * ( l.blue * m.blue ) ) };
	
	return color;
}

Color colorNormalize( Color c1 )
{
	Color color= { c1.red/255.0, c1.green/255.0, c1.blue/255.0 };

	return color;
}

int colorIsParecidas( Color c1 , Color c2)
{
	double k1 = abs(c1.red - c2.red);
	double k2 = abs(c1.blue - c2.blue);
	double k3 = abs(c1.green - c2.green);
	double r = MAX(k1,MAX(k2,k3));
	if(r > 5.0/255.0) return 0;
	else return 1;
}

void colorGetOGLVector(Color ambientLight,float* tmp)
{
    tmp[0] = ambientLight.red;
    tmp[1] = ambientLight.green;
    tmp[2] = ambientLight.blue;
    tmp[3] = 1.0;
}

