/**
 *	@file color.h Color: manipulao de cores.
 *
 *	@author 
 *			- Maira Noronha
 *			- Thiago Bastos
 *          
 *
 *	@date
 *			Criado em:			01 de Dezembro de 2002
 *			ltima Modificao:	22 de Janeiro de 2003
 *
 *	@version 2.0
 */

#ifndef	_COLOR_H_
#define	_COLOR_H_

/************************************************************************/
/* Tipos Exportados                                                     */
/************************************************************************/
/**
 *   Cor rgb.
 */
typedef struct color_impl
{
   /**
    * Componente vermelha.
    */
	double red;
   /**
    * Componente verde.
    */
	double green;
   /**
    * Componente azul.
    */
	double blue;
}
Color;

/************************************************************************/
/* Funes Exportadas                                                   */
/************************************************************************/
/**
 *	Cria uma cor a partir de 3 bytes RGB.
 */
Color colorCreate3b( unsigned char red, unsigned char green, unsigned char blue );

/**
 *	Soma duas cores.
 */
Color colorAddition( Color c1, Color c2 );

Color colorContribute( Color c1, Color c2 );

/**
 *	Amplia ou reduz o brilho de uma cor, multiplicando-o por um fator.
 */
Color colorScale( double scale, Color color );

/**
 *	Multiplica duas cores.
 *	til para obter a cor de um material iluminado por uma luz ambiente.
 */
Color colorMultiplication( Color c1, Color c2 );

/**
 *	Cor refletida por um material iluminado por uma certa luz, usando um
 *	coeficiente especificado (scale). til nos clculos de iluminao.
 */
Color colorReflection( double scale, Color light, Color material );

/**
 *	Torna a faixa de valores de r, g e b entre 0 e 1.
 */
Color colorNormalize( Color color );

Color colorAdd( Color c1, Color c2 );

int colorIsParecidas( Color c1 , Color c2);

void colorGetOGLVector(Color ambientLight,float* tmp);

#endif

