/**
 *	@file light.h Light: definio e operaes com fontes de luz.
 *
 *	@author 
 *			- Maira Noronha
 *			- Thiago Bastos
 *
 *	@date
 *			Criado em:			01 de Dezembro de 2002
 *			ltima Modificao:	22 de Janeiro de 2003
 *
 *	@version 2.0
 */

#ifndef _LIGHT_H_
#define _LIGHT_H_

#include "color.h"
#include "algebra.h"


/************************************************************************/
/* Tipos Exportados                                                     */
/************************************************************************/
/**
 *   Luz com posio e intensidade.
 */
struct _Light
{
    /**
     *  Posio da luz.
     */
	Vector position;
	/**
     *  Intensidade da luz em rgb.
     */
	Color color;
};

typedef struct _Light * Light;

/************************************************************************/
/* Funes Exportadas                                                   */
/************************************************************************/
/**
 *	Cria uma fonte de luz no-direcional com as propriedades especificadas.
 *
 *	@param position Posio da fonte de luz.
 *	@param color Cor da luz.
 *
 *	@return Handle para a fonte de luz.
 */
Light lightCreate( Vector position, Color color );

/**
 *	Obtm a posio em que est localizada uma fonte de luz.
 *
 *	@param light Fonte de luz.
 *
 *	@return Posio da fonte de luz.
 */
Vector lightGetPosition( Light light );

/**
 *	Obtm a cor de uma fonte de luz.
 *
 *	@param light Fonte de luz.
 *
 *	@return Cor da fonte de luz.
 */
Color lightGetColor( Light light );

/**
 *	Destri uma fonte de luz criada com lightCreate().
 *
 *	@param light Fonte de luz.
 */
void lightDestroy( Light light );

/**
 *	Posiciona uma fonte de luz.
 *
 *	@param light Fonte de luz.
 *	@param position Posio da fonte de luz.
 */
void lightSetPosition( Light light, Vector position );

/**
 *	Fixa a cor de uma fonte de luz.
 *
 *	@param light Fonte de luz.
 *	@param color Cor da fonte de luz.
 */
void lightSetColor( Light light, Color color );

#endif

