/*
*	Computao Grfica - Trabalho de Raytracing
*	Professor Marcelo Gattass
*
*	@file main.c 
*
*	@author 
*			- Maira Noronha
*			- Thiago Bastos
*
*	@date
*			ltima Modificao:		10 de Fevereiro de 2003
*
*	@version 2.0
*
*	Este arquivo no precisa ser modificado.
*	Veja o mdulo raytracing.c
*/

#include <stdio.h>
#include <time.h>

/*- Inclusao das bibliotecas IUP e CD: ------------------------------------*/
#include <iup.h>                    /* interface das funcoes IUP */
#include <iupgl.h>                  /* interface da funcao do canvas do OpenGL no IUP */
#include <windows.h>                /* inclui as definicoes do windows para o OpenGL */
#include <gl/gl.h>                  /* prototypes do OpenGL */
#include <gl/glu.h>                 /* prototypes do OpenGL */


#include "image.h"                  /* TAD para imagem */
#include "color.h"
#include "algebra.h"
#include "raytracing.h"
#include "fila.h"
#include "zbuffer.h"
#include "camera.h"
#include "mainaux.h"
/* -- implemented in "iconlib.c" to load standard icon images into IUP */
void IconLibOpen(void);

/*- Contexto do Programa: -------------------------------------------------*/
Scene scene;         /* cena corrente */
int width = -1 ; 
int height=-1; /* alrgura e altura corrente */
Image image;         /* imagem que armazena o resultado at agora do algoritmo */
Vector eyeG;
Camera cameraG;
Fila* filaRay; /*Fila do progressivo*/
int yc;

int zbuffer;

Ihandle *canvas;      /* ponteiro IUP dos canvas */
Ihandle *label;       /* ponteiro IUP do label para colocar mensagens para usuario */

/*- Funcoes auxiliares ------------*/

/* Dialogo de selecao de arquivo  */
static char * get_file_name( void )
{
	Ihandle* getfile = IupFileDlg();
	char* filename = NULL;

	IupSetAttribute(getfile, IUP_TITLE, "Abertura de arquivo"  );
	IupSetAttribute(getfile, IUP_DIALOGTYPE, IUP_OPEN);
	IupSetAttribute(getfile, IUP_FILTER, "*.rt4");
	IupSetAttribute(getfile, IUP_FILTERINFO, "Arquivo de cena (*.rt4)");
	IupPopup(getfile, IUP_CENTER, IUP_CENTER);  /* o posicionamento nao esta sendo respeitado no Windows */

	filename = IupGetAttribute(getfile, IUP_VALUE);
	return filename;
}


/*------------------------------------------*/
/* Callbacks do IUP.                        */
/*------------------------------------------*/


/* - Callback de mudanca de tamanho no canvas (mesma para ambos os canvas) */
int resize_cb(Ihandle *self, int widthL, int heightL)
{
	IupGLMakeCurrent(self);  /* torna o foco do OpenGL para este canvas */

	/* define a area do canvas que deve receber as primitivas do OpenGL */
	glViewport(0,0,width,height);

	/* transformacao de instanciacao dos objetos no sistema de coordenadas da camera */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();           /* identidade,  ou seja nada */

	/* transformacao de projecao */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D (0.0, (GLsizei)(width), 0.0, (GLsizei)(height));  /* ortografica no plano xy de [0,w]x[0,h] */

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	//glClear(GL_COLOR_BUFFER_BIT);

	return IUP_DEFAULT; /* retorna o controle para o gerenciador de eventos */
}


/* - Callback de repaint do canvas  */
int repaint_cb(Ihandle *self)
{
	int w,h;
	int x,y;
	Color rgb;


	if (yc!=height || zbuffer) return IUP_DEFAULT; /* esta callback so'desenha depois que o algoritmo termina a imagem */
	imageGetDimensions(image,&w,&h);
	IupGLMakeCurrent(canvas);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glBegin(GL_POINTS);
	for (y=0;y<h;y++) {
		for (x=0; x<w; x++) {
			rgb=imageGetPixel(image, x, y);
			glColor3f((float)rgb.red,(float)rgb.green,(float)rgb.blue);
			glVertex2i(x,y);
		}
	}
	glEnd();
    glFlush();
	IupGLSwapBuffers(canvas);
	

	return IUP_DEFAULT; /* retorna o controle para o gerenciador de eventos */
} 


/* captura um nome para um arquivo */
static char * get_new_file_name( void )
{
	Ihandle* getfile = IupFileDlg();
	char* filename = NULL;

	IupSetAttribute(getfile, IUP_TITLE, "Salva arquivo"  );
	IupSetAttribute(getfile, IUP_DIALOGTYPE, IUP_SAVE);
	IupSetAttribute(getfile, IUP_FILTER, "*.tga");
	IupSetAttribute(getfile, IUP_FILTERINFO, "Arquivo de imagem (*.tga)");
	IupPopup(getfile, IUP_CENTER, IUP_CENTER);  /* o posicionamento nao esta sendo respeitado no Windows */

	filename = IupGetAttribute(getfile, IUP_VALUE);
	return filename;
}

/* salva a imagem gerada */
int save_cb(Ihandle *self)
{
	char* filename = get_new_file_name( );  /* chama o dialogo de abertura de arquivo */
	if (filename==NULL) return 0;
	imageWriteTGA(filename,image);
	return IUP_DEFAULT;
}

void linhaLinha(void)
{
	int x;
	
	/* Faz uma linha de pixels por vez */
	if (yc<height)
 {
		IupGLMakeCurrent(canvas);
		glBegin(GL_POINTS);
		for( x = 0; x < width; ++x )
  {
			Color pixel = { 0.0, 0.0, 0.0 };
			Vector ray;				
			Color color;

   
   //ray tracer distribuido
   if(RTDistr)
   {
				int xi, yi, numAmostras = 4;
				double xRandom, yRandom, pixelsCelula, totalX, totalY, deltaX, deltaY = 0;
				
				pixelsCelula = 1.0 / numAmostras;				
				for(yi = 0; yi < numAmostras; yi++)
    {
					deltaX = 0;
					for(xi = 0; xi < numAmostras; xi++)
     {
						// gera raio nessa clula
						xRandom = rand() * pixelsCelula / ((double)RAND_MAX + 1);
						yRandom = rand() * pixelsCelula / ((double)RAND_MAX + 1);

      totalX = x + deltaX + xRandom;
      totalY = yc + deltaY + yRandom;

						ray = camGetRay( cameraG, totalX, totalY);

      color = rayTrace( scene, eyeG, ray, 0, globalIlumination);
					 pixel = colorContribute( pixel, color );

						deltaX += pixelsCelula;
					}
					deltaY += pixelsCelula;
				}

				pixel = colorScale(1.0/(numAmostras*numAmostras), pixel);

    color.red = 0.0;
    color.green = 0.0;
    color.blue = 0.0;
    pixel = colorAddition( pixel, color );
   }
   else
   {
			 ray = camGetRay( cameraG, x, yc );
			 			
			 /* Obtm a amostra. */
			 color = rayTrace( scene, eyeG, ray, 0, globalIlumination );

			 /* Adiciona a contribuio da amostra  cor final */
			 pixel = colorAddition( pixel, color );
   }

   imageSetPixel( image, x, yc, pixel );
			glColor3f((float)pixel.red,(float)pixel.green,(float)pixel.blue);
			glVertex2i(x,yc);
		}
		glEnd();
		yc++;
		glFlush();
	}
}

int idle_Zbuffer_cb(void)
{
	//Chama a zbuffer
	
	IupGLMakeCurrent(canvas);	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();	
	ZBufferScene(scene,width,height);
	IupGLSwapBuffers(canvas);
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	return IUP_DEFAULT;
}

/* calcula uma linha da imagem a cada camada de idle */
int idle_cb(void)
{
	IupGLMakeCurrent(canvas);
	if(yc < height){
			linhaLinha();
			IupGLSwapBuffers(canvas);
	}
	else{
		IupSetFunction (IUP_IDLE_ACTION, (Icallback) NULL); /* a imagem ja' esta' completa */
	}
	
	return IUP_DEFAULT;
}

/* Trata os eventos de teclado */
int keybord_cb(Ihandle *self, int c, int press)
{
	Vector eye = eyeG;
	Camera camera = cameraG;
	Vector up = algUnit(camera->up);
	Vector direcao  = algSub(cameraG->at,eye);
	Matrix rotacao;
	Vector temp,terceiroEixo = algUnit(algCross( direcao, up));
	
	if(zbuffer == TRUE){
		if((c ==  K_w) && press )
		{		
			eye = algAdd(eye,algScale(2,algUnit(direcao)));
			camera->at = algAdd(camera->at,algScale(2,algUnit(direcao)));
		}
		if((c ==  K_s) && press )
		{		
			eye = algAdd(eye,algScale(2,algMinus(algUnit(direcao))));
			camera->at = algAdd(camera->at,algScale(2,algMinus(algUnit(direcao))));
		}	    
		if((c ==  K_LEFT) && press )
		{	
		    rotacao = algMatrixRotate(2,up.x,up.y,up.z);			
			temp = algTransf(rotacao,direcao);			
			camera->at = algAdd(temp,eye);		
		}
		if((c ==  K_RIGHT) && press )
		{		
			rotacao = algMatrixRotate(-2,up.x,up.y,up.z);			
			temp = algTransf(rotacao,direcao);			
			camera->at = algAdd(temp,eye);	
		}
		if((c ==  K_d) && press )
		{	
            eye = algAdd(eye,algScale(2,terceiroEixo));
            camera->at = algAdd(camera->at,algScale(2,terceiroEixo));	
			
		}
		if((c ==  K_a) && press )
		{		
            eye = algAdd(eye,algScale(2,algMinus(terceiroEixo)));
            camera->at = algAdd(camera->at,algScale(2,algMinus(terceiroEixo)));
		}
		if((c ==  K_UP) && press )
		{					
            rotacao = algMatrixRotate(2,terceiroEixo.x,terceiroEixo.y,terceiroEixo.z);			
            temp = algTransf(rotacao,direcao);			
            camera->at = algAdd(temp,eye);		
		}
		if((c ==  K_DOWN) && press )
		{		
            rotacao = algMatrixRotate(-2,terceiroEixo.x,terceiroEixo.y,terceiroEixo.z);			
            temp = algTransf(rotacao,direcao);			
            camera->at = algAdd(temp,eye);			
		}
	}
	eyeG = eye;
	camera->eye = eye;
	cameraG = camera;
    scene->camera = camera;
	
	return IUP_DEFAULT;
}


/* callback que define o tipo de exibicao */

int linha_cb(void)
{
	yc = 0 ;	
	zbuffer = FALSE;
	resize_cb(canvas,width,height);
	scene->camera = camCreate( scene->camera->eye, scene->camera->at, scene->camera->up, scene->camera->fovy, scene->camera->nearp, scene->camera->farp, scene->camera->screenWidth, scene->camera->screenHeight );
	camDestroy(cameraG); 
	cameraG = scene->camera;
	IupSetFunction (IUP_IDLE_ACTION, (Icallback) idle_cb);
	IupSetFocus(canvas);
	return IUP_DEFAULT;
}

int rtDistr_cb(Ihandle* self)
{
 if(RTDistr)
  RTDistr = 0;
 else
  RTDistr = 1;

 return IUP_DEFAULT;
}

int ilumGlobal_cb(Ihandle* self)
{
 if(globalIlumination)
  globalIlumination = 0;
 else
  globalIlumination = 1;

 return IUP_DEFAULT;
}

int ray_cb(void)
{	
		zbuffer = TRUE;
		IupSetFunction (IUP_IDLE_ACTION, (Icallback) idle_Zbuffer_cb);
	
	IupSetFocus(canvas);
	return IUP_DEFAULT;
}


//int inicializaTipo(int tipo);
///* carrega uma nova cena */
//void minimizadorFar()
//{
//	int i;
//	int maiorZ;
//	for(i = 0;i < scene->objectCount;i++)
//	{
//        scene->objects;		
//	}
//}

int load_cb(void) {
	char* filename = get_file_name();  /* chama o dialogo de abertura de arquivo */
	char buffer[30];

	if (filename==NULL) return 0;

	/* Le a cena especificada */
	scene = sceLoad( filename );
	if( scene == NULL ) return IUP_DEFAULT;

	cameraG = sceGetCamera( scene );
	eyeG = camGetEye( cameraG );
	width = camGetScreenWidth( cameraG );
	height = camGetScreenHeight( cameraG );
	if (image) imageDestroy(image);
	image = imageCreate( width, height );
	IupSetfAttribute(label, "TITLE", "%3dx%3d", width, height);
	IupSetFunction (IUP_IDLE_ACTION, (Icallback) idle_Zbuffer_cb );

	sprintf(buffer,"%3dx%3d", width, height);
	IupSetAttribute(canvas,IUP_RASTERSIZE,buffer);
	free(filaRay);
	filaRay = fila_alloc();
	yc = 0 ;

	IupSetFocus(canvas);
	return IUP_DEFAULT;
}


/*-------------------------------------------------------------------------*/
/* Incializa o programa.                                                   */
/*-------------------------------------------------------------------------*/

int init(void)
{
	Ihandle *dialog, *statusbar,  *box ;

	Ihandle *toolbar, *load, *save , *linha , *distr, *ilumGlobal, *foto;

	/* creates the toolbar and its buttons */
	load = IupButton("", "load_cb");
	IupSetAttribute(load,"TIP","Carrega uma imagem.");
	IupSetAttribute(load,"IMAGE","icon_lib_open");
	IupSetFunction("load_cb", (Icallback)load_cb);

	save = IupButton("", "save_cb");
	IupSetAttribute(save,"TIP","Salva no formato TGA.");
	IupSetAttribute(save,"IMAGE","icon_lib_save");

	linha = IupButton("RayTrace","linha_cb");
	IupSetFunction("linha_cb", (Icallback)linha_cb);
 IupSetAttribute(linha,"TIP","Linha a linha");

 distr = IupToggle("Distribuido","rtDistr_cb");
	IupSetFunction("rtDistr_cb", (Icallback)rtDistr_cb);
 IupSetAttribute(distr,"TIP","Ray Trace Distribuido");

 ilumGlobal = IupToggle("Iluminao Global","ilumGlobal_cb");
	IupSetFunction("ilumGlobal_cb", (Icallback)ilumGlobal_cb);
 IupSetAttribute(ilumGlobal,"TIP","Ray Trace com Iluminao Global");

	foto = IupButton("OpenGL","ray_cb");
	IupSetAttribute(foto,"TIP","Modo Zbuffer");
	IupSetFunction("ray_cb", (Icallback)ray_cb);


	toolbar = IupHbox(
		load, 
		save,
		foto,
  linha,
		distr,
  ilumGlobal,
		IupFill(),
		NULL);

	IupSetAttribute(toolbar, "ALIGNMENT", "ACENTER");
	IupSetAttribute(toolbar, "GAP", "5");

	/* cria um canvas */
	canvas = IupGLCanvas("repaint_cb"); 
	IupSetAttribute(canvas,IUP_RASTERSIZE,"500x500");
	IupSetAttribute(canvas, "RESIZE_CB", "resize_cb");
	IupSetAttribute(canvas, IUP_KEYPRESS_CB,IUP_KEYPRESS_CB);
	IupSetAttribute(canvas, IUP_BUFFER, "DOUBLE");

	/* associa o evento de repaint a funccao repaint_cb */
	IupSetFunction("repaint_cb", (Icallback) repaint_cb);
	IupSetFunction("save_cb", (Icallback)save_cb);
	IupSetFunction("resize_cb", (Icallback) resize_cb);
	IupSetFunction (IUP_IDLE_ACTION, (Icallback) NULL); 

	IupSetFunction(IUP_KEYPRESS_CB, (Icallback)keybord_cb);

	/* the status bar is just a label to put some usefull information in run time */
	label = IupLabel("status");
	IupSetAttribute(label, "EXPAND", "HORIZONTAL");
	IupSetAttribute(label, "FONT", "COURIER_NORMAL_10");
	statusbar = IupSetAttributes(IupHbox(
		IupFrame(IupHbox(label, NULL)), 
		NULL), "MARGIN=5x5");

	/* this is the most external box that puts together
	the toolbar, the two canvas and the status bar */

	box = IupVbox(
		toolbar,
		canvas, 
		statusbar, 
		NULL);

	/* create the dialog and set its attributes */
	dialog = IupDialog(box);
	IupSetAttribute(dialog, "CLOSE_CB", "app_exit_cb");
	IupSetAttribute(dialog, "TITLE", "FCG - Ray Tracing");

	zbuffer = TRUE;

 RTDistr = 0;
 globalIlumination = 0;

	IupShowXY(dialog, IUP_CENTER, IUP_CENTER);
//NAVEGACAO
#if 0  
	piover180 = 0.0174532925f;
	heading = 0;//;
	xpos = 0;
	zpos = 0;
	yrot = 30;				// Y Rotation
	walkbias = 0;
	walkbiasangle = 0;
	lookupdown  = 10.0f;
	z  =0.0f;				// Depth Into The Screen
	xtrans  = 0;//-xpos;
	ztrans  = 0;//-zpos;
	ytrans  = -0.25f;//-walkbias-0.25f;
	sceneroty  = 330.f;//360.0f - yrot;
#else
	piover180 = 0.0;
	heading = 0;//;
	xpos = 0;
	zpos = 0;
	yrot = 0;
	walkbias = 0;
	walkbiasangle = 0;
	lookupdown  = 0;
	z  =0.0f;				// Depth Into The Screen
	xtrans  = 0;//-xpos;
	ztrans  = 0;//-zpos;
	ytrans  = 0;
	sceneroty  = 0;
#endif
	return 1;
}

/*-------------------------------------------------------------------------*/
/* Rotina principal.                                                       */
/*-------------------------------------------------------------------------*/
void main(void) {
	IupOpen();
	IupGLCanvasOpen();
	IconLibOpen();
	filaRay = fila_alloc();
	if ( init() )
		IupMainLoop();
	IupClose();
	free(filaRay);
}
