/**
 *	@file zbuffer.c RayTracing: renderizao de cenas pelo OpenGL.
 *
 *	@author 
 *			- Marcelo Gattass
 *			- Thiago Bastos
 *
 *	@date
 *			Criado em:             10 de novembro de 2003
 *
 *	@version 1.0
 */

#include <stdlib.h>
#include <gl/glut.h>
#include "camera.h"
#include "algebra.h"
#include "color.h"
#include "scene.h"
#include "zbuffer.h"
#include "object.h"
#include <GL/gl.h>


void ZBufferScene(Scene scene, int winW, int winH )
{
    Camera camera;
     
	double fovy, aspect, zNear, zFar; 

	/* parametros externos da camera (correspondentes a glLookAt) */
    double eyex, eyey, eyez, centerx, centery, centerz, upx, upy, upz;

	/* parametros internos da camera (correspondentes a gluPerspective) */
	int numberOfLights;
	/* Luz, objetos e acao... */
	int numberOfObjects;
	int i;

    glPushAttrib(GL_ENABLE_BIT | GL_LIGHTING_BIT | GL_TRANSFORM_BIT | GL_CURRENT_BIT | GL_COLOR_BUFFER_BIT);

 	camera = sceGetCamera(scene);

	numberOfLights  = sceGetLightCount(scene);
	numberOfObjects = sceGetObjectCount(scene);

    /* inicializa o OpenGL */
    glEnable     (GL_DEPTH_TEST);  /* habilita o teste de profundidade do z-buffer */
    glDepthFunc  (GL_LESS);        /* menor z significa mais proximo */
    glEnable     (GL_LIGHTING);    /* ativa os calculos de iluminacao */
    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);  /* reflexao especular com relacao ao olho */
	/*modifica o clculo dos ngulos de reflexo especular - observador local ou infinito*/

    glShadeModel (GL_SMOOTH);      /* suavisa as diferencas de cores entre os objetos */
    glEnable     (GL_NORMALIZE);   /* normaliza os vetores normais antes de utilizar */    
	glClearColor(scene->bgColor.red,scene->bgColor.green,scene->bgColor.blue,0.5f);
	glClear      (GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT ); /* limpa o buffer de profundidade */

    /* Define a matriz de projecao */
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glViewport(0,0,winW,winH); /* mapeamento */
	camGetOGLPerspective(camera, &fovy, &aspect, &zNear, &zFar); /* pega os parametros da cena */
    gluPerspective(fovy, aspect, zNear, zFar); /* aspect = largura / altura */

	/* define a matriz do modelo e da transformacao para a camera */
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity();
	camGetOGLLookAt(camera, &eyex,&eyey,&eyez, &centerx,&centery,&centerz, &upx,&upy,&upz);
	gluLookAt(eyex, eyey, eyez, centerx, centery, centerz, upx, upy, upz); /* gera a matriz de view */

	/* define as luzes no sistema de eixos dos modelos globais */
	for( i = 0; i < numberOfLights; ++i ) 
	{
 	 Light  light          = sceGetLight( scene, i );
	 Vector lightPosition  = lightGetPosition(light);
	 Color  lightColor     = lightGetColor(light);
     Color  ambientLight   = sceGetAmbientLight(scene);

	 float tmp[4];
	 
	 colorGetOGLVector(ambientLight,tmp);      
	     glLightfv(GL_LIGHT0+i, GL_AMBIENT, tmp);
     colorGetOGLVector(lightColor,tmp); 
	     glLightfv(GL_LIGHT0+i, GL_DIFFUSE, tmp);
         glLightfv(GL_LIGHT0+i, GL_SPECULAR,tmp);
     algGetOGLVector(lightPosition,tmp); 
	     glLightfv(GL_LIGHT0+i, GL_POSITION,tmp);
	 glEnable(GL_LIGHT0+i);
	}
	
	/* desenha os objetos */
	for (i=0;i < numberOfObjects; i++ )
	{
 	 Object object          = sceGetObject(scene,i);
	 Material material      = sceGetMaterial(scene,objGetMaterial(object));
	 Color diffuseColor     = matGetDiffuse(material, algVector(0,0,0,1));
	 Color specularColor    = matGetSpecular(material);
	 double specularExpoent = matGetSpecularExponent(material);
	
	 float tmp[4],zero[]={0.0,0.0,0.0,1.0};
	 colorGetOGLVector(diffuseColor,tmp);
	   glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, tmp);
	 colorGetOGLVector(specularColor,tmp);
       glMaterialfv(GL_FRONT, GL_SPECULAR, tmp);
     glMaterialf(GL_FRONT, GL_SHININESS, specularExpoent );
     glMaterialfv(GL_FRONT, GL_EMISSION, zero);
     
     //if (material->texturaZbuffer != NULL)
     //{
     //   glEnable(GL_TEXTURE_2D);                        
     //   glBindTexture(GL_TEXTURE_2D, material->texturaZbuffer->texID);
     //}
     
     
     glColor4f(1.0f,1.0f,1.0f,material->opacityFactor);

     // Funo de combinao para Translucncia baseada na fonte 
     glBlendFunc(GL_SRC_ALPHA,GL_ONE);
     if(object->blend)                   
         {
            glEnable(GL_BLEND);       // Liga o Blending
            glDisable(GL_DEPTH_TEST); // Desliga o teste de profundidade
         }
     else                        // Por outro lado
         {
            glDisable(GL_BLEND);      // Desliga o Blending
            glEnable(GL_DEPTH_TEST);  // Liga o teste de profundiade
     }


	 /* cada objeto deve se instaciar */
	 glPushMatrix();
	   objDraw(object);
     glPopMatrix();
	
    //if (material->texturaZbuffer != NULL)
    //{
    //    glDisable(GL_TEXTURE_2D);                        
    //}
    }
  glFlush();
  
  glPopAttrib();
}


